@@ -2,7 +2,7 @@ use crate::converter::CTypeConverter;
 use crate::converter::Converter;
 use crate::Config;
 use interoptopus::indented;
-use interoptopus::lang::c::{CType, CompositeType, Constant, Documentation, EnumType, Field, FnPointerType, Function, OpaqueType, Variant};
+use interoptopus::lang::c::{CType, CompositeType, Constant, EnumType, Field, FnPointerType, Function, OpaqueType, Variant};
 use interoptopus::patterns::TypePattern;
 use interoptopus::util::sort_types_by_dependencies;
 use interoptopus::writer::IndentWriter;
@@ -61,18 +61,10 @@ pub trait CWriter {
     }
 
     fn write_function(&self, w: &mut IndentWriter, function: &Function) -> Result<(), Error> {
-        self.write_function_declaration(w, function)
+        self.write_function_declaration(w, function, 999)
     }
 
-    fn write_documentation(&self, w: &mut IndentWriter, documentation: &Documentation) -> Result<(), Error> {
-        for line in documentation.lines() {
-            indented!(w, r#"/// {}"#, line)?;
-        }
-
-        Ok(())
-    }
-
-    fn write_function_declaration(&self, w: &mut IndentWriter, function: &Function) -> Result<(), Error> {
+    fn write_function_declaration(&self, w: &mut IndentWriter, function: &Function, max_line: usize) -> Result<(), Error> {
         let attr = &self.config().function_attribute;
         let rval = self.converter().to_type_specifier(function.signature().rval());
         let name = self.converter().function_name_to_c_name(function);
@@ -90,18 +82,31 @@ pub trait CWriter {
             }
         }
 
-        indented!(w, r#"{}{} {}({});"#, attr, rval, name, params.join(", "))
+        // Test print line to see if we need to break it
+        let line = format!(r#"{}{} {}({});"#, attr, rval, name, params.join(", "));
+
+        if line.len() <= max_line {
+            indented!(w, r#"{}{} {}({});"#, attr, rval, name, params.join(", "))
+        } else {
+            indented!(w, r#"{}{} {}("#, attr, rval, name)?;
+            for p in params {
+                indented!(w, [_], r#"{}"#, p)?;
+            }
+            indented!(w, [_], r#");"#)
+        }
     }
 
     fn write_type_definitions(&self, w: &mut IndentWriter) -> Result<(), Error> {
+        let mut known_function_pointers = vec![];
+
         for the_type in &sort_types_by_dependencies(self.library().ctypes().to_vec()) {
-            self.write_type_definition(w, the_type)?;
+            self.write_type_definition(w, the_type, &mut known_function_pointers)?;
         }
 
         Ok(())
     }
 
-    fn write_type_definition(&self, w: &mut IndentWriter, the_type: &CType) -> Result<(), Error> {
+    fn write_type_definition(&self, w: &mut IndentWriter, the_type: &CType, known_function_pointers: &mut Vec<String>) -> Result<(), Error> {
         match the_type {
             CType::Primitive(_) => {}
             CType::Array(_) => {}
@@ -118,7 +123,7 @@ pub trait CWriter {
                 w.newline()?;
             }
             CType::FnPointer(f) => {
-                self.write_type_definition_fn_pointer(w, f)?;
+                self.write_type_definition_fn_pointer(w, f, known_function_pointers)?;
                 w.newline()?;
             }
             CType::ReadPointer(_) => {}
@@ -126,7 +131,7 @@ pub trait CWriter {
             CType::Pattern(p) => match p {
                 TypePattern::AsciiPointer => {}
                 TypePattern::NamedCallback(e) => {
-                    self.write_type_definition_fn_pointer(w, e.fnpointer())?;
+                    self.write_type_definition_fn_pointer(w, e.fnpointer(), known_function_pointers)?;
                     w.newline()?;
                 }
                 TypePattern::FFIErrorEnum(e) => {
@@ -152,11 +157,11 @@ pub trait CWriter {
         Ok(())
     }
 
-    fn write_type_definition_fn_pointer(&self, w: &mut IndentWriter, the_type: &FnPointerType) -> Result<(), Error> {
-        self.write_type_definition_fn_pointer_body(w, the_type)
+    fn write_type_definition_fn_pointer(&self, w: &mut IndentWriter, the_type: &FnPointerType, known_function_pointers: &mut Vec<String>) -> Result<(), Error> {
+        self.write_type_definition_fn_pointer_body(w, the_type, known_function_pointers)
     }
 
-    fn write_type_definition_fn_pointer_body(&self, w: &mut IndentWriter, the_type: &FnPointerType) -> Result<(), Error> {
+    fn write_type_definition_fn_pointer_body(&self, w: &mut IndentWriter, the_type: &FnPointerType, known_function_pointers: &mut Vec<String>) -> Result<(), Error> {
         let rval = self.converter().to_type_specifier(the_type.signature().rval());
         let name = self.converter().fnpointer_to_typename(the_type);
 
@@ -165,7 +170,14 @@ pub trait CWriter {
             params.push(format!("{} x{}", self.converter().to_type_specifier(param.the_type()), i));
         }
 
-        indented!(w, "typedef {} (*{})({});", rval, name, params.join(", "))
+        let fn_pointer = format!("typedef {} (*{})({});", rval, name, params.join(", "));
+
+        if !known_function_pointers.contains(&fn_pointer) {
+            indented!(w, "{}", fn_pointer)?;
+            known_function_pointers.push(fn_pointer);
+        }
+
+        Ok(())
     }
 
     fn write_type_definition_enum(&self, w: &mut IndentWriter, the_type: &EnumType) -> Result<(), Error> {
