@@ -11,13 +11,21 @@ fn generate_bindings(output: &str) -> Result<(), Error> {
         ..Config::default()
     };
 
-    Generator::new(config, interoptopus_reference_project::ffi_inventory()).write_file(output)
+    let library = interoptopus_reference_project::ffi_inventory();
+
+    let generator = Generator::new(config, library);
+    generator.write_file(format!("{}/my_header.h", output))?;
+
+    // let doc_gen = DocGenerator::new(library, generator);
+    // doc_gen.write_file(format!("{}/my_header.md", output))?;
+
+    Ok(())
 }
 
 #[test]
 #[cfg_attr(miri, ignore)]
 fn bindings_match_reference() -> Result<(), Error> {
-    generate_bindings("tests/output/my_header.h")?;
+    generate_bindings("tests/output/")?;
 
     assert_file_matches_generated("tests/output/my_header.h");
 
@@ -27,7 +35,8 @@ fn bindings_match_reference() -> Result<(), Error> {
 #[test]
 #[cfg_attr(miri, ignore)]
 fn bindings_work() -> Result<(), Error> {
-    generate_bindings("tests/output/my_header.h")?;
+    generate_bindings("tests/output/")?;
+
     compile_c_app_if_installed("tests/output/", "tests/output/app.c")?;
     Ok(())
 }
